/** Matrix.h
 *
 *  generic/float matrix data structure
 *
 *  C. Kessler reworked 991202
 */

typedef struct { 
  int nrows;       // number of rows
  int ncols;       // number of columns (or max. #cols if sparse)
  float **row;     // array of pointers to first element of each row
  float *data;     // points to linearized array holding all elements
#ifdef SPARSE
  int *n;          // array of sizes of #nonzeros per row (if sparse) 
  int nz;          // number of stored nonzero elements (sparse)
  int *col;        // array of column indices for the nonzeros (sparse)
#endif
} matrix, *Matrix;

// constructors:
extern sync Matrix new_Matrix( sh int nr, sh int nc, sh int theheap );
//extern sync Matrix copyMatrix( sh Matrix M );
extern sync Matrix **splitMatrix( sh Matrix M, sh int x, sh int y );
// destructor:
extern void freeMatrix( Matrix M );
// output:
extern void printMatrix( Matrix M );
// accessing single matrix elements:
//macro for void Set( Matrix M, int i, int j, float value ):
#define Set( M, i, j, value ) \
           (M)->row[i][j] = (value)
// macro for float Get( Matrix M, int i, int j ):
#define Get( M, i, j ) \
           ((M)->row[i][j])
// arithmetics:
extern sync void Add( sh Matrix A, sh Matrix B, sh Matrix C );
extern sync void Sub( sh Matrix A, sh Matrix B, sh Matrix C );
extern sync void Mul( sh Matrix A, sh Matrix B, sh Matrix C );  // standard method
extern sync void StrassenMul( sh Matrix A, sh Matrix B, sh Matrix C );

