/** Implementation of the taskQueue skeleton function in Fork95 
 *  @author C. Kessler
 *  @date 990811
 */

#include <fork.h>
#include <io.h>
#include <math.h>
#include <../util/PQueue.h>
#include <../util/TaskQueue.h>

/** The taskQueue skeleton function:
 *  the implementation uses the parallel FIFO queue PQueue
 */

// The spawn macro is defined in <../util/TaskQueue.h>:
// #define spawn( taskpool, task ) \
//         PQueuePut( (PQueue)(taskpool), (void *)(task) )

sync void taskQueue(
  sh sync void (*presetTaskQueue)( sh TaskQueue, sh void * ),
  sh void (*thetask)( TaskQueue, void * ),
  sh void *presetparam )
{
  sh PQueue taskpool;
  sh int p = groupsize();
  sh int nr_procs_still_working = p;
  int was_idle = 0;
  taskpool = new_PQueue( p );
  presetTaskQueue( taskpool, presetparam );
  farm {
     // now try to get a task and execute it:
     do {
        void *t = PQueueGet( taskpool );
#if DEBUG
        Task tt = (Task)t;
        if (tt) pprintf("Got task ([%d,%d,%d,%d],%d,%d)\n",
                tt->col[0],
                (tt->row > 1) ? tt->col[1]:-1,
                (tt->row > 2) ? tt->col[2]:-1,
                (tt->row > 3) ? tt->col[3]:-1,
                tt->row, tt->pos );
#endif
        if (!t) {
          if (!was_idle) {
             syncadd( &nr_procs_still_working, -1 );
             was_idle = 1;
          }
        }
        else {
          if (was_idle) {
             syncadd( &nr_procs_still_working, 1 );  // nr of not idle procs
             was_idle = 0;
          }
          thetask( taskpool, t );
        }
     }
     while ( nr_procs_still_working > 0 
            || !(taskpool->empty(taskpool))) ;
  }
}

